// Note: These preprocessor directives are overwritten during loading into fuel. Don't overwrite these.
#shadowsamplers

#if (__VERSION__ >= 330)
#	define HIGHP
#	define MEDIUMP
#	define LOWP
#	define FRAGOUT out vec4 glfrag;
#	define FRAG glfrag
#	define OUT out
#	define IN smooth in
#	define TEXTURE texture
#   define SHADOW2DPROJ(tex, coord) textureProj( tex, coord )
#   define SHADOWSAMPLER sampler2DShadow
#else
#	define HIGHP highp
#	define MEDIUMP mediump
#	define LOWP lowp
#	define TEXTURE texture2D
#	define FRAGOUT
#	define FRAG gl_FragColor
#	define OUT varying
#	define IN varying
#   if USE_SHADOW_SAMPLERS
#       extension GL_EXT_shadow_samplers : enable
#       define SHADOW2DPROJ(tex, coord) shadow2DProjEXT( tex, coord )
#       define SHADOWSAMPLER sampler2DShadow
#   else
#       define SHADOW2DPROJ(tex, coord) texture2DProj(tex, coord).x
#       define SHADOWSAMPLER sampler2D
#   endif
#endif

LOWP vec4 brightenColor( LOWP vec4 color )
{
	color.rgb *= 2.0;
	return color;
}

LOWP mat2 rotMatFromCycle( in HIGHP float cycle )
{
	HIGHP float si	= sin(cycle);
	HIGHP float co	= cos(cycle);
	return mat2( co, si, -si, co );
}

HIGHP float computeDistance( in HIGHP vec4 position_objSpace, in HIGHP mat4 modelmatrix, in HIGHP vec4 eyeposition )
{
	HIGHP vec4 vtxpos = modelmatrix * position_objSpace;
	HIGHP float dist = distance( vtxpos, eyeposition );
	return dist;
}

LOWP float computeFog( in HIGHP vec4 fogdef, in HIGHP float dist )
{
	HIGHP float fogdistance = fogdef.x;
	HIGHP float fogstart = fogdef.y;
	HIGHP float fogband = fogdef.z;
	HIGHP float fogposition = ( dist - fogstart) / fogband;
	LOWP float fogopacity = 1.0 - clamp(fogposition, 0.0, 1.0);
	return fogopacity;
}

HIGHP vec4 curvature(in HIGHP vec4 _pos, in HIGHP vec2 _curvature)
{
	_pos.x = _pos.x + (_pos.z * _pos.z * 0.0020 * _curvature.x);
	_pos.y = _pos.y - (_pos.z * _pos.z * 0.0005 * _curvature.y);
	return _pos;
}

HIGHP mat3 espTranspose(in HIGHP mat3 inMatrix)
{
    HIGHP vec3 i0 = inMatrix[0];
    HIGHP vec3 i1 = inMatrix[1];
    HIGHP vec3 i2 = inMatrix[2];
    
    HIGHP mat3 outMatrix = mat3(
        vec3(i0.x, i1.x, i2.x),
        vec3(i0.y, i1.y, i2.y),
        vec3(i0.z, i1.z, i2.z));
    return outMatrix;
}

HIGHP vec2 uv1Offset( in HIGHP vec2 _uvstart, in HIGHP vec2 _uvoff, in HIGHP vec2 _wh, in HIGHP vec2 _xy )
{
	HIGHP vec2 outuv = _uvstart + _uvoff;
	HIGHP vec2 FrameXYRounded = floor( _xy );
	outuv += (FrameXYRounded * _wh );
	return outuv;
}

LOWP vec3 espSaturation(in LOWP vec3 inColor, in HIGHP float _sat)
{
    HIGHP float grey_s = (inColor.r * 0.3) + (inColor.g * 0.59) + (inColor.b * 0.11);
    LOWP vec3 grey_c = vec3(grey_s);
    LOWP vec3 out_c = mix(grey_c, inColor, _sat);
    return out_c;
}
